@@ -29,7 +29,7 @@ from idds.orm.base import models
 
 def create_request(scope=None, name=None, requester=None, request_type=None, transform_tag=None,
                    status=RequestStatus.New, locking=RequestLocking.Idle, priority=0,
-                   lifetime=30, workload_id=None, request_metadata=None,
+                   lifetime=None, workload_id=None, request_metadata=None,
                    processing_metadata=None):
     """
     Create a request.
@@ -68,10 +68,15 @@ def create_request(scope=None, name=None, requester=None, request_type=None, tra
             request_metadata = {}
         request_metadata['is_pseudo_input'] = True
 
+    if lifetime:
+        expired_at = datetime.datetime.utcnow() + datetime.timedelta(days=lifetime)
+    else:
+        expired_at = None
+
     new_request = models.Request(scope=scope, name=name, requester=requester, request_type=request_type,
                                  transform_tag=transform_tag, status=status, locking=locking,
                                  priority=priority, workload_id=workload_id,
-                                 expired_at=datetime.datetime.utcnow() + datetime.timedelta(days=lifetime),
+                                 expired_at=expired_at,
                                  request_metadata=request_metadata, processing_metadata=processing_metadata)
     return new_request
 
@@ -79,7 +84,7 @@ def create_request(scope=None, name=None, requester=None, request_type=None, tra
 @transactional_session
 def add_request(scope=None, name=None, requester=None, request_type=None, transform_tag=None,
                 status=RequestStatus.New, locking=RequestLocking.Idle, priority=0,
-                lifetime=30, workload_id=None, request_metadata=None,
+                lifetime=None, workload_id=None, request_metadata=None,
                 processing_metadata=None, session=None):
     """
     Add a request.
@@ -215,6 +220,22 @@ def get_requests(request_id=None, workload_id=None, with_detail=False, with_meta
             if with_metadata:
                 query = session.query(models.Request)\
                                .with_hint(models.Request, "INDEX(REQUESTS REQUESTS_SCOPE_NAME_IDX)", 'oracle')
+
+                if request_id:
+                    query = query.filter(models.Request.request_id == request_id)
+                if workload_id:
+                    query = query.filter(models.Request.workload_id == workload_id)
+
+                tmp = query.all()
+                rets = []
+                if tmp:
+                    for t in tmp:
+                        if to_json:
+                            t_dict = t.to_dict_json()
+                        else:
+                            t_dict = t.to_dict()
+                        rets.append(t_dict)
+                return rets
             else:
                 query = session.query(models.Request.request_id,
                                       models.Request.scope,
@@ -235,26 +256,18 @@ def get_requests(request_id=None, workload_id=None, with_detail=False, with_meta
                                       models.Request.errors)\
                                .with_hint(models.Request, "INDEX(REQUESTS REQUESTS_SCOPE_NAME_IDX)", 'oracle')
 
-            if request_id:
-                query = query.filter(models.Request.request_id == request_id)
-            if workload_id:
-                query = query.filter(models.Request.workload_id == workload_id)
+                if request_id:
+                    query = query.filter(models.Request.request_id == request_id)
+                if workload_id:
+                    query = query.filter(models.Request.workload_id == workload_id)
 
-            tmp = query.all()
-            rets = []
-            if tmp:
-                for t in tmp:
-                    if to_json:
-                        t_dict = t.to_dict_json()
-                    else:
-                        t_dict = t.to_dict()
-                    if not with_metadata:
-                        if 'request_metadata' in t_dict:
-                            del t_dict['request_metadata']
-                        if 'processing_metadata' in t_dict:
-                            del t_dict['processing_metadata']
-                    rets.append(t_dict)
-            return rets
+                tmp = query.all()
+                rets = []
+                if tmp:
+                    for t in tmp:
+                        t2 = dict(zip(t.keys(), t))
+                        rets.append(t2)
+                return rets
         else:
             subquery1 = session.query(models.Collection.coll_id, models.Collection.transform_id,
                                       models.Collection.scope.label("input_coll_scope"),
