@@ -287,8 +287,9 @@ def get_transforms(request_id=None, workload_id=None, transform_id=None, workpro
         raise error
 
 
-@read_session
-def get_transforms_by_status(status, period=None, locking=False, bulk_size=None, to_json=False, by_substatus=False, session=None):
+@transactional_session
+def get_transforms_by_status(status, period=None, transform_ids=[], locking=False, locking_for_update=False,
+                             bulk_size=None, to_json=False, by_substatus=False, only_return_id=False, session=None):
     """
     Get transforms or raise a NoObject exception.
 
@@ -309,19 +310,28 @@ def get_transforms_by_status(status, period=None, locking=False, bulk_size=None,
         if len(status) == 1:
             status = [status[0], status[0]]
 
-        query = session.query(models.Transform)
+        if only_return_id:
+            query = session.query(models.Transform.transform_id)
+        else:
+            query = session.query(models.Transform)
+
         if by_substatus:
             query = query.filter(models.Transform.substatus.in_(status))
         else:
             query = query.filter(models.Transform.status.in_(status))
         query = query.filter(models.Transform.next_poll_at < datetime.datetime.utcnow())
 
+        if transform_ids:
+            query = query.filter(models.Transform.transform_id.in_(transform_ids))
         if period:
             query = query.filter(models.Transform.updated_at < datetime.datetime.utcnow() - datetime.timedelta(seconds=period))
         if locking:
             query = query.filter(models.Transform.locking == TransformLocking.Idle)
 
-        query = query.order_by(asc(models.Transform.updated_at)).order_by(desc(models.Transform.priority))
+        if locking_for_update:
+            query = query.with_for_update(skip_locked=True)
+        else:
+            query = query.order_by(asc(models.Transform.updated_at)).order_by(desc(models.Transform.priority))
 
         if bulk_size:
             query = query.limit(bulk_size)
@@ -330,10 +340,13 @@ def get_transforms_by_status(status, period=None, locking=False, bulk_size=None,
         rets = []
         if tmp:
             for t in tmp:
-                if to_json:
-                    rets.append(t.to_dict_json())
+                if only_return_id:
+                    rets.append(t[0])
                 else:
-                    rets.append(t.to_dict())
+                    if to_json:
+                        rets.append(t.to_dict_json())
+                    else:
+                        rets.append(t.to_dict())
         return rets
     except sqlalchemy.orm.exc.NoResultFound as error:
         raise exceptions.NoObject('No transforms attached with status (%s): %s' %
@@ -361,6 +374,19 @@ def update_transform(transform_id, parameters, session=None):
                                                                TransformStatus.Failed, TransformStatus.Failed.value]:
             parameters['finished_at'] = datetime.datetime.utcnow()
 
+        if 'transform_metadata' in parameters and 'work' in parameters['transform_metadata']:
+            work = parameters['transform_metadata']['work']
+            if work is not None:
+                work.refresh_work()
+                if 'running_metadata' not in parameters:
+                    parameters['running_metadata'] = {}
+                parameters['running_metadata']['work_data'] = work.metadata
+        if 'transform_metadata' in parameters:
+            del parameters['transform_metadata']
+        if 'running_metadata' in parameters:
+            parameters['_running_metadata'] = parameters['running_metadata']
+            del parameters['running_metadata']
+
         session.query(models.Transform).filter_by(transform_id=transform_id)\
                .update(parameters, synchronize_session=False)
     except sqlalchemy.orm.exc.NoResultFound as error:
